const electron = require('electron');
const { ipcMain } = require('electron');
// Module to control application life.
const app = electron.app;
// Module to create native browser window.
const BrowserWindow = electron.BrowserWindow;
const working_path = process.cwd();
const path = require('path');
const url = require('url');
const machineUuid = require("machine-uuid");
const { spawn } = require('child_process');
const request = require('request');
const fs = require("fs");

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;

let myWindow = null;

const gotTheLock = app.requestSingleInstanceLock();

if (!gotTheLock) {
    app.quit();
} else {
    app.on('second-instance', (event, commandLine, workingDirectory) => {
        if (myWindow) {
            if (myWindow.isMinimized()) myWindow.restore();
            myWindow.focus();
        }
    })
}

function createWindow() {
    // Create the browser window.
    mainWindow = new BrowserWindow({
        width: 1024,
        height: 768,
        frame: false,
        kiosk: false,
        webPreferences: {
            nodeIntegration: true
        }
    })

    // and load the index.html of the app.
    mainWindow.loadURL(url.format({
        pathname: path.join(__dirname, 'www/index.html'),
        protocol: 'file:',
        slashes: true
    }))

    // Open the DevTools.
    // mainWindow.webContents.openDevTools();
    mainWindow.maximize();

    let webContents = mainWindow.webContents;
    webContents.on('did-finish-load', () => {
        webContents.setZoomFactor(1);
        webContents.setVisualZoomLevelLimits(1, 1);
        webContents.setLayoutZoomLevelLimits(0, 0);
    });

    // Emitted when the window is closed.
    mainWindow.on('closed', function () {
        // Dereference the window object, usually you would store windows
        // in an array if your app supports multi windows, this is the time
        // when you should delete the corresponding element.
        mainWindow = null
    });

    // Receive update event from frontend, start to download
    ipcMain.on("updateEvent", (event, arg) => {
        downloadFileWorker(arg, working_path + "/resources");
    });

    machineUuid().then((device_uuid)=>{
      global.device_info = {
        uuid: device_uuid
      }
    })

}

function update() {
    var downloaded = working_path + '/resources/update.zip';
    var updateAsar = working_path + '/resources/update.asar';
    var appAsar = working_path + '/resources/app.asar';
    let executable = process.execPath;
    const WindowsUpdater = working_path + "/resources//updater.exe";

    try {
        if (!fs.existsSync(downloaded)) return;
        // rename downloaded file first
        fs.renameSync(downloaded, updateAsar);
        winArgs = `${JSON.stringify(WindowsUpdater)} ${JSON.stringify(updateAsar)} ${JSON.stringify(appAsar)} ${JSON.stringify(executable)}`
        spawn('cmd', ['/s', '/c', '"' + winArgs + '"'], {
            detached: true,
            windowsVerbatimArguments: true,
            stdio: 'ignore'
        })
        app.quit();
    }
    catch (err) {
        console.error(err);
    }
}

function downloadFileWorker(patchUrl, baseDir) {

    const downloadFile = 'update.zip';

    let receivedBytes = 0;
    let totalBytes = 0;

    const req = request({
        method: 'GET',
        uri: patchUrl
    });

    const out = fs.createWriteStream(path.join(baseDir, downloadFile));
    req.pipe(out);

    req.on('response', (data) => {
        // get total bytes
        totalBytes = parseInt(data.headers['content-length'], 10);
    });

    req.on('data', (chunk) => {
        // get downloaded file size
        receivedBytes += chunk.length;
        showProgress(receivedBytes, totalBytes);
    });

    req.on('end', () => {
        update();
    });
}

function showProgress(received, total) {
    const percentage = (received * 100) / total;
    // send progress to frontend
    mainWindow.webContents.send('downloadProgress', percentage.toFixed(0));
}

app.commandLine.appendSwitch('touch-events', 'enabled');
// app.commandLine.appendSwitch('js-flags', '--max-nex-space-size=1024');
// app.commandLine.appendSwitch('js-flags', '--max-old-space-size=300');
// app.commandLine.appendSwitch('js-flags', '--expose-gc');

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow)





// Quit when all windows are closed.
app.on('window-all-closed', function () {
    // On OS X it is common for applications and their menu bar
    // to stay active until the user quits explicitly with Cmd + Q
    if (process.platform !== 'darwin') {
        app.quit()
    }
})

app.on('activate', function () {
    // On OS X it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (mainWindow === null) {
        createWindow()
    }
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.
